@@ -5,6 +5,8 @@ use warnings;
 use strict;
 use 5.014;
 
+use Clone qw( clone );
+
 use Term::Choose         qw();
 use Term::Form::ReadLine qw();
 
@@ -37,7 +39,7 @@ sub case {
             case_info => $opt->{info} // $ax->get_sql_info( $sql )
         };
     }
-    my $tmp_sql = $ax->backup_href( $sql );
+    my $tmp_sql = clone( $sql );
     $tmp_sql->{case_stmt} = $r_data->{case}[-1] // '';
     $tmp_sql->{case_info} = $r_data->{case_info};
     my $in = ' ' x $sf->{o}{G}{base_indent};
@@ -92,7 +94,7 @@ sub case {
             return;
         }
         push @bu, $tmp_sql->{case_stmt};
-        my $op = '=';
+        my $operator= '=';
         if ( $menu->[$idx] eq $end ) {
             $tmp_sql->{case_stmt} .= "\n${pad1}END";
             my $case_stmt = delete $tmp_sql->{case_stmt};
@@ -105,9 +107,9 @@ sub case {
             return $case_stmt;
         }
         elsif ( $menu->[$idx] eq $when ) {
-            my $stmt = 'when_stmt';
-            $tmp_sql->{$stmt} = "${pad2}WHEN";
-            my $ret = $sb->__add_condition( $tmp_sql, $clause, $stmt, $qt_cols );
+            my $substmt_type = "${pad2}WHEN";
+            my $ret = $sb->__add_condition( $tmp_sql, $clause, $substmt_type, $qt_cols );
+
             if ( ! defined $ret ) {
                 delete $tmp_sql->{when_stmt};
                 $tmp_sql->{case_stmt} = pop @bu;
@@ -116,7 +118,7 @@ sub case {
             $tmp_sql->{case_stmt} .= "\n" . delete $tmp_sql->{when_stmt};
             $tmp_sql->{case_stmt} .= " THEN";
             push @{$r_data->{case}}, $tmp_sql->{case_stmt};
-            my $value = $ext->value( $tmp_sql, $clause, $r_data, $op );
+            my $value = $ext->value( $tmp_sql, $clause, $r_data, $operator);
             pop @{$r_data->{case}};
             if ( ! defined $value ) {
                 $tmp_sql->{case_stmt} = pop @bu;
@@ -127,7 +129,7 @@ sub case {
         elsif ( $menu->[$idx] eq $else ) {
             $tmp_sql->{case_stmt} .= "\n${pad2}ELSE";
             push @{$r_data->{case}}, $tmp_sql->{case_stmt};
-            my $value = $ext->value( $tmp_sql, $clause, $r_data, $op );
+            my $value = $ext->value( $tmp_sql, $clause, $r_data, $operator);
             pop @{$r_data->{case}};
             if ( ! defined $value ) {
                 $tmp_sql->{case_stmt} = pop @bu;
